<?php

namespace Drupal\spammaster;

use Drupal\Core\Database\Connection;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxy;
use GuzzleHttp\Client;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class service.
 */
class SpamMasterElusiveService {
  use StringTranslationTrait;

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The session account interface.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The String Translation..
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $stringTranslation;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The SpamMasterElusiveService service.
   *
   * @var \Drupal\spammmaster\SpamMasterElusiveService
   */
  protected $spamelusive;

  /**
   * The SpamMasterCollectService Service.
   *
   * @var \Drupal\spammaster\SpamMasterCollectService
   */
  protected $collectService;

  /**
   * The SpamMasterUserService Service.
   *
   * @var \Drupal\spammaster\SpamMasterUserService
   */
  protected $userService;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, MessengerInterface $messenger, AccountInterface $account, RequestStack $requestStack, Client $httpClient, ConfigFactoryInterface $configFactory, StateInterface $state, TranslationInterface $stringTranslation, AccountProxy $current_user, SpamMasterCollectService $collectService, SpamMasterUserService $userService) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->account = $account;
    $this->requestStack = $requestStack;
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->state = $state;
    $this->stringTranslation = $stringTranslation;
    $this->currentUser = $current_user;
    $this->collectService = $collectService;
    $this->userService = $userService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('request_stack'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('string_translation'),
      $container->get('current_user'),
      $container->get('spammaster.collect_service'),
      $container->get('spammaster.user_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterElusiveCheck($spamelusive) {

    $this->spamelusive = $spamelusive;
    $spammaster_date = date("Y-m-d H:i:s");
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_license = $spammaster_settings->get('spammaster.license_key');
    $spammaster_status = $this->state->get('spammaster.license_status');
    $spammaster_total_block_count = $this->state->get('spammaster.total_block_count');
    if ('VALID' === $spammaster_status || 'MALFUNCTION_1' === $spammaster_status || 'MALFUNCTION_2' === $spammaster_status) {

      // Call collect service.
      $collectnow = $this->requestStack->getCurrentRequest()->getClientIp();
      $spammaster_collect_service = $this->collectService;
      $is_collected = $spammaster_collect_service->spamMasterGetCollect($collectnow);
      $spamcollection = json_decode($is_collected);
      $spammasterip = $spamcollection->spammasterip;
      $spammasteragent = $spamcollection->spammasteragent;
      $spammasterreferer = $spamcollection->spammasterreferer;
      $spammasterurl = $spamcollection->spammasterurl;

      // User service call.
      $spamuser = $this->currentUser->id();
      $spammaster_user_service = $this->userService;
      $is_user = $spammaster_user_service->spamMasterGetUser($spamuser);
      $spamuserdata = json_decode($is_user);
      $spammasteruserid = $spamuserdata->ID;
      $spammasterusername = $spamuserdata->username;

      // Exempt whitelist.
      $spammaster_white_query = $this->connection->select('spammaster_white', 'u');
      $spammaster_white_query->fields('u', ['white']);
      $spammaster_white_query->where('(white = :ip)', [
        ':ip' => $spammasterip,
      ]);
      $spammaster_white_result = $spammaster_white_query->execute()->fetchObject();
      // White positive, or administrator log insert.
      $user_roles = $this->account->getRoles();
      if (!empty($spammaster_white_result) || in_array('administrator', $user_roles) || 'none' !== $spammasteruserid) {
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-firewall',
          'spamvalue' => 'Spam Master: form submission within whitelist or registered user, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
        ])->execute();
        return 'BAIL';
      }
      else {
        // Buffer db check.
        $spammaster_spam_buffer_query = $this->connection->select('spammaster_threats', 'u');
        $spammaster_spam_buffer_query->fields('u', ['threat']);
        $spammaster_spam_buffer_query->where('(threat = :ip)', [
          ':ip' => $spammasterip,
        ]);
        $spammaster_spam_buffer_result = $spammaster_spam_buffer_query->execute()->fetchObject();
        // Buffer db positive, throw error, log insert.
        if (!empty($spammaster_spam_buffer_result)) {
          $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
          $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);

          $this->connection->insert('spammaster_keys')->fields([
            'date' => $spammaster_date,
            'spamkey' => 'spammaster-firewall',
            'spamvalue' => 'Spam Master: form submission buffer block, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
          ])->execute();
          return 'ELUSIVE';
        }
        else {
          // Exclusions by value.
          $spam_action_array = [
            '1',
            'Add+to+cart',
            'Checkout',
            'Continue+as+Guest',
            'Continue+to+review',
            'Pay+and+complete+purchase',
          ];
          $spammaster_drupalajax = $spamelusive->get('_drupal_ajax');
          $spammaster_op = $spamelusive->get('op');
          $spammaster_ids = $spamelusive->get('ids');
          if (isset($spammaster_drupalajax) || isset($spammaster_op) || isset($spammaster_ids)) {
            if (in_array($spammaster_drupalajax, $spam_action_array, TRUE) || !empty($spammaster_op) || !empty($spammaster_ids)) {
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: drupal_ajax form submission within post exclusion, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              $spamproceed = '';
              return 'BAIL';
            }
            else {
              $spamproceed = 'PROC';
            }
          }
          else {
            $spamproceed = 'PROC';
          }
          // Exclude malformed internal links.
          $spammasterautodisc = 'https://appserver-e9c36b99-nginx-8a47c893ed644cd1b1b2aa564c46d7aa/autodiscover/autodiscover.xml';
          if (isset($spammasterautodisc)) {
            if ($spammasterautodisc == $spammasterurl) {
              $spamproceed = '';
              return 'BAIL';
            }
            else {
              $spamproceed = 'PROC';
            }
          }
          else {
            $spamproceed = 'PROC';
          }
          if ('PROC' === $spamproceed) {
            // Web api check. Create data to be posted.
            $blog_threat_type = 'HAF';
            $blog_threat_content = 'HAF';
            // Collect email to scan.
            $params = [];
            foreach ($spamelusive as $key => $value) {
              if ($key[0] !== '_') {
                $params[$key] = $value;
              }
            }
            if (preg_grep('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $params)) {
              $matches = preg_grep('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $params);
              if ($matches) {
                foreach ($matches as $key => $val) {
                  $resultKey = $key;
                  if ($resultKey) {
                  }
                  if (filter_var($val, FILTER_VALIDATE_EMAIL)) {
                    $spammasteremail = substr($val, 0, 256);
                  }
                  else {
                    $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
                  }
                }
              }
              else {
                $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
              }
            }
            else {
              $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
            }
            $blog_web_address = $this->requestStack->getCurrentRequest()->getHost();
            $address_unclean = $blog_web_address;
            $address = preg_replace('#^https?://#', '', $address_unclean);
            @$blog_server_ip = $_SERVER['SERVER_ADDR'];
            // If empty ip.
            if (empty($blog_server_ip) || $blog_server_ip == '0') {
              @$blog_server_ip = 'I ' . gethostbyname($_SERVER['SERVER_NAME']);
            }
            $spam_master_leaning_url = 'aHR0cHM6Ly93d3cuc3BhbW1hc3Rlci5vcmcvd3AtY29udGVudC9wbHVnaW5zL3NwYW0tbWFzdGVyLWFkbWluaXN0cmF0b3IvaW5jbHVkZXMvbGVhcm5pbmcvZ2V0X2xlYXJuX2hhZi5waHA=';

            // Call drupal hhtpclient.
            $client = $this->httpClient;
            // Post data.
            $request = $client->post(base64_decode($spam_master_leaning_url), [
              'form_params' => [
                'blog_license_key'    => $spammaster_license,
                'blog_threat_ip'      => $spammasterip,
                'blog_threat_user'    => $is_user,
                'blog_threat_type'    => $blog_threat_type,
                'blog_threat_email'   => $spammasteremail,
                'blog_threat_content' => $blog_threat_content,
                'blog_threat_agent'   => $spammasteragent,
                'blog_threat_refe'    => $spammasterreferer,
                'blog_threat_dest'    => $spammasterurl,
                'blog_web_adress'     => $address,
                'blog_server_ip'      => $blog_server_ip,
              ],
            ]);
            // Decode json data.
            $response = json_decode($request->getBody(), TRUE);
            if (empty($response['threat'])) {
              // Log clean rbl check.
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: form submission rbl check, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              return 'BAIL';
            }
            else {
              // Insert ip into buffer db.
              $this->connection->insert('spammaster_threats')->fields([
                'date' => $spammaster_date,
                'threat' => $spammasterip,
              ])->execute();
              // Web positive, return.
              $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
              $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: form submission rbl block, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              return 'ELUSIVE';
            }
          }
        }
      }
    }
    else {
      return 'BAIL';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterIsJson($spamelusive) {

    $this->spamelusive = $spamelusive;
    $spammaster_status = $this->state->get('spammaster.license_status');
    $spammaster_total_block_count = $this->state->get('spammaster.total_block_count');
    if ('VALID' === $spammaster_status || 'MALFUNCTION_1' === $spammaster_status || 'MALFUNCTION_2' === $spammaster_status) {

      if (is_string($spamelusive) && is_array(json_decode($spamelusive, TRUE)) && (json_last_error() === JSON_ERROR_NONE)) {
        $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
        $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);

        // Decode json.
        $spam_elusive_decoded = json_decode($spamelusive, TRUE);
        foreach ($spam_elusive_decoded as $val) {
          if (preg_grep('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $val)) {
            $matches = preg_grep('/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$/', $val);
            if ($matches) {
              foreach ($matches as $val) {
                // Return $val inside loop to spam master api.
                return 'BAIL';
              }
            }
            else {
              return 'BAIL';
            }
          }
          else {
            return 'BAIL';
          }
        }
      }
      else {
        return 'ELUSIVE';
      }
    }
    else {
      return 'BAIL';
    }
  }

}
